# coding=utf-8
import json
from collections import OrderedDict
from axxac.path_tool import *


# 加载itest中导出的目标json格式的文件
# json_file：传入一个从itest中导出的json文件
def read_itest_json(json_file):
    if not os.path.isfile(json_file):
        return None
    else:
        file_data = open(json_file, encoding='utf-8')
        return json.load(file_data)


def get_header_list(header):
    header_list = []
    for header_item in header:
        header_list.append(
            {
                "name": str(header_item[0]),
                "value": str(header_item[1]),
                "enable": True,
                "encode": True,
                "file": False,
                "required": True,
                "valid": True,
                "contentType": "",
                "description": ""
            }
        )
    return header_list


def get_extract_list(extract):
    extract_list = []
    for header_item in extract:
        header_item.append(
            {
                "type": "JSONPath",
                "variable": str(header_item[0]),
                "value": str(header_item[1]),
                "expression": "获取"+str(header_item[0])
            }
        )
    return extract_list


def get_assert_list(the_assert):
    assert_list = []
    for assert_item in the_assert:
        assert_list.append(
            {
                "type": "JSON",
                "expression": str(assert_item[0]),
                "expect": str(assert_item[1]),
                "description": str(assert_item[2])
            }
        )
    return assert_list


def generate_header_item(row):
    request_item = OrderedDict([
        ("contentType",""),
        ("description", ""),
        ("enable",True),
        ("encode", True),
        ("file", False),
        ("name", row[0]),
        ("required",True),
        ( "valid", True),
        ("value",  row[1])
    ])
    return request_item


def generate_extract_item(row):
    extract_item = OrderedDict([
        ("type","JSONPath"),
        ("variable",row[0]),
        ("value", row[1]),
        ("expression","获取"+row[0])
    ])
    return extract_item


def generate_extract_item_null():
    extract_item = OrderedDict([
        ("type","JSONPath"),
        ("variable",""),
        ("value", ""),
        ("expression","")
    ])
    return extract_item


def is_json_str(object):
    try:
        json.loads(object)
    except ValueError:
        return False
    return True


def generate_request_item(header, method, title, row, the_assert, extract):
    raw = dict()
    querys_items = []
    for i in range(6, len(row)):
        value = row[i]
        if is_json_str(value):
            value = json.loads(value) # 如果是json字符串，转成json对象
        raw[title[i]] = value

    if method == "POST":
        query_item2 = OrderedDict([
            ("contentType", "text/plain"),
            ("enable", True),
            ("encode", True),
            ("file", False),
            ("required", True),
            ("type", "text"),
            ("valid", False),
            ("name", ""),
            ("value", "")
        ])
        querys_items.append(query_item2)
    else:
        raw = ""
        for i in range(6, len(row)):
            qname = title[i]
            qvalue = row[i]
            query_item1 = OrderedDict([
                ("contentType", "text/plain"),
                ("enable", True),
                ("encode", True),
                ("file", False),
                ("required", True),
                ("type", "text"),
                ("valid", False),
                ("name", qname),
                ("value", qvalue)
            ])
            querys_items.append(query_item1)





    assert_list = get_assert_list(the_assert)
    # 每一个header
    header_items = []
    for row_num in range(0, len(header)):
        header_item = generate_header_item(header[row_num])
        header_items.append(header_item)

    # 每一个提取
    extract_items = []
    for row_num in range(0, len(extract)):
        extract_item = generate_extract_item_null()
        extract_items.append(extract_item)

    item1 = OrderedDict([
        ("name",row[2]),
        ("priority",row[5]),
        ("request",{
        "type": "HTTPSamplerProxy",
        "hashTree": [
            {
                "type": "Assertions",
                "text": [

                ],
                "regex": [

                ],
                "jsonPath": assert_list,
                 "jsr223": [

                      ],
                "xpath2": [

                ],
                "duration": {
                "type": "Duration"
                },
                "enable": True,
                "name": "断言的名字",
                "active": True
            },
            {
                "type": "Extract",
                "regex": [

                ],
                "json": extract_items,
                "xpath": [

                ],
                "enable": True,
                "name": "提取的名字",
                "active": True
            }
        ],
        "customizeReq": False,
        "headers": header_items,
        "body": {
            "raw": json.dumps(raw, indent=4, ensure_ascii=False),
            "type": "JSON",
            "valid": True,
            "xml": False

        },
        "followRedirects": False,
        "doMultipartPost": False,
        "arguments": querys_items
    })
    ])



    # itest平台原因，需要保持request顺序才能正常解析
    item = OrderedDict([
        ("type", "HTTP"),
        ("name", row[2]),
        ("enable", True),
        ("assertions", {
            "jsonPath": assert_list
        }),
        ("url", row[1]),
        ("method", method),
        ("parameters",method ),
        ("headers", [{
            "enable": True,
            "encode": True,
            "file": False,
            "required": True,
            "valid": False
        }]),
        ("body", {
            "type": "Raw",
            "raw": json.dumps(raw, indent=4, ensure_ascii=False),
            "kvs": [{
                "type": "text",
                "enable": True,
                "contentType": "text/plain",
                "encode": True,
                "file": False,
                "required": True,
                "valid": False
            }],
            "format": "json",
            "json": False,
            "kV": False,
            "oldKV": True,
            "valid": False,
            "xml": False
        })
    ])
    return item1


def generate(result_json, output_dir):
    method = result_json['method']
    header = result_json['header']
    case_list = result_json['case_list']
    normal_assert = result_json['normal_assert']
    fail_assert = result_json['fail_assert']
    extract = result_json['extract']
    # 读取itest导出的json文件
    itest_json = read_itest_json(get_cur_dir()+'/itest_template.json')



    # 每一条case
    case_items = list()

    for row_num in range(1, len(case_list)):
        is_normal = row_num == 1
        the_assert = normal_assert if is_normal else fail_assert
        case_item = generate_request_item(header, method, case_list[0], case_list[row_num], the_assert, extract)
        case_items.append(case_item)
        itest_json = case_items # itest格式item数组下只取1个元素

    # 写文件
    itest_json_str = json.dumps(itest_json, indent=4, ensure_ascii=False)
    output_dir = get_output_dir(output_dir)
    path = output_dir +'/itest.json'
    with open(path, 'w', encoding='utf-8') as file:
        file.write(itest_json_str)
    print("iTest JSON文件已生成："+os.path.abspath(path))

    return itest_json



# coding=utf-8
from axxac.generate_case_list import generate as clg
from axxac.generate_itest_json import generate as ijg
from axxac.require import generate_require_items

def execute(input_dir, output_dir):
    case_result_json = clg(input_dir, output_dir)
    ijg(case_result_json, output_dir)


# coding=utf-8
import xlrd


def get_name(sheet):
    return sheet.row_values(1)[0]


def get_url(sheet):
    return sheet.row_values(1)[1]


def get_method(sheet):
    return sheet.row_values(1)[2]


def get_query(sheet):
    query_rows = []
    for row_num in range(1, sheet.nrows):
        row_values = sheet.row_values(row_num)
        query_rows.append(row_values)
    return query_rows


def get_header(sheet):
    header_rows = []
    for row_num in range(1, sheet.nrows):
        row_values = sheet.row_values(row_num)
        header_rows.append(row_values)
    return header_rows


def get_body(sheet):
    body = []
    for col_num in range(sheet.ncols):
        col_values = sheet.col_values(col_num)
        remove_tail_space(col_values) # 删除每列最后由于列值个数不同而自动生成的空字符串
        param = dict()
        param['name'] = col_values[0]
        param['values'] = col_values[1:]
        body.append(param)
    return body


def remove_tail_space(array):
    new_array = array[:]
    for i in range(len(new_array)-1, -1, -1):
        if new_array[i] == '':
            array.pop()
        else:
            break


def get_assert(sheet):
    assert_rows = []
    for row_num in range(1, sheet.nrows):
        row_values = sheet.row_values(row_num)
        assert_rows.append(row_values)
    return assert_rows


def get_extract(sheet):
    extract_rows = []
    for row_num in range(1, sheet.nrows):
        row_values = sheet.row_values(row_num)
        extract_rows.append(row_values)
    return extract_rows


def read_excel(excel_file):
    data = xlrd.open_workbook(excel_file)
    cfg_sheet = data.sheet_by_index(0)
    header_sheet = data.sheet_by_index(1)
    body_sheet = data.sheet_by_index(2)
    assert_normal_sheet = data.sheet_by_index(3)
    assert_fail_sheet = data.sheet_by_index(4)
    extract_sheet = data.sheet_by_index(5)

    result_json = dict()
    result_json["name"] = get_name(cfg_sheet)
    result_json["url"] = get_url(cfg_sheet)
    result_json["method"] = get_method(cfg_sheet)
    result_json["header"] = get_header(header_sheet)
    result_json["body"] = get_body(body_sheet)
    result_json["normal_assert"] = get_assert(assert_normal_sheet)
    result_json["fail_assert"] = get_assert(assert_fail_sheet)
    result_json["extract"] = get_extract(extract_sheet)
    return result_json



# coding=utf-8
import xlrd
import os


def get_name(sheet):
    return sheet.row_values(1)[0]


def get_url(sheet):
    return sheet.row_values(1)[1]


def get_method(sheet):
    return sheet.row_values(1)[2]


def get_query(sheet):
    query_rows = []
    for row_num in range(1, sheet.nrows):
        row_values = sheet.row_values(row_num)
        query_rows.append(row_values)
    return query_rows


def get_header(sheet):
    header_rows = []
    for row_num in range(1, sheet.nrows):
        row_values = sheet.row_values(row_num)
        header_rows.append(row_values)
    return header_rows


def get_body(sheet):
    body_rows = []
    for row_num in range(1, sheet.nrows):
        row_values = sheet.row_values(row_num)
        body_rows.append(row_values)
    return body_rows

def remove_tail_space(array):
    new_array = array[:]
    for i in range(len(new_array)-1, -1, -1):
        if new_array[i] == '':
            array.pop()
        else:
            break


def get_assert(sheet):
    assert_rows = []
    for row_num in range(1, sheet.nrows):
        row_values = sheet.row_values(row_num)
        assert_rows.append(row_values)
    return assert_rows

def get_extract(sheet):
    extract_rows = []
    for row_num in range(1, sheet.nrows):
        row_values = sheet.row_values(row_num)
        extract_rows.append(row_values)
    return extract_rows

def get_config_value(sheet):
    row_values = sheet.row_values(1)
    return row_values


def get_header_value(sheet):
    header_rows = []
    for row_num in range(1, sheet.nrows):
        row_values = sheet.row_values(row_num)
        header_rows.append(row_values)
    return header_rows


def get_token_value(sheet):
    token_rows = []
    for row_num in range(1, sheet.nrows):
        row_values = sheet.row_values(row_num)
        token_rows.append(row_values)
    return token_rows


def get_assert_value(sheet):
    assert_rows = []
    for row_num in range(1, sheet.nrows):
        row_values = sheet.row_values(row_num)
        assert_rows.append(row_values)
    return assert_rows


def read_excel(requires_dir):
    if not os.path.isdir(requires_dir):
        return None

    result_json_array = []
    items = os.listdir(requires_dir)
    items.sort(key=lambda x: int(x.split('-')[0])) # 升序排序
    for item in items:
        path = os.path.join(requires_dir, item)
        if os.path.isfile(path):
            data = xlrd.open_workbook(path)
            cfg_sheet = data.sheet_by_index(0)
            query_sheet = data.sheet_by_index(1)
            header_sheet = data.sheet_by_index(2)
            body_sheet = data.sheet_by_index(3)
            assert_normal_sheet = data.sheet_by_index(4)
            assert_fail_sheet = data.sheet_by_index(5)
            extract_sheet = data.sheet_by_index(6)

            result_json = dict()
            result_json["name"] = get_name(cfg_sheet)
            result_json["url"] = get_url(cfg_sheet)
            result_json["method"] = get_method(cfg_sheet)
            result_json["query"] = get_query(query_sheet)
            result_json["header"] = get_header(header_sheet)
            result_json["body"] = get_body(body_sheet)
            result_json["normal_assert"] = get_assert(assert_normal_sheet)
            result_json["fail_assert"] = get_assert(assert_fail_sheet)
            result_json["extract"] = get_extract(extract_sheet)

            result_json_array.append(result_json)

    return result_json_array



# coding=utf-8
import sys
import getopt
from axxac.auto_case import execute

HELP = 'Usage:\n  axxac -i <input_directory> -o <output_directory>'


def run(argv, help_info):
    argv = argv[1:]
    if len(argv) == 0:
        print(help_info)
        sys.exit()
    input_dir = ''
    output_dir = ''
    try:
        opts, args = getopt.getopt(argv, "hi:o:")
    except getopt.GetoptError:
        print(help_info)
        sys.exit()
    for opt, arg in opts:
        if opt == '-h':
            print(help_info)
            sys.exit()
        elif opt == "-i":
            input_dir = arg
        elif opt == "-o":
            output_dir = arg

    execute(input_dir, output_dir)


if __name__ == '__main__':
    execute("../data", "../output/")
    # run(sys.argv, HELP)

def cmdexe():
    run(sys.argv, HELP)


from axxac import read_require_config
import json
from collections import OrderedDict


def get_header_list(header):
    header_list = []
    for header_item in header:
        header_list.append(
            {
                "name": str(header_item[0]),
                "value": str(header_item[1]),
                "enable": True,
                "encode": True,
                "file": False,
                "required": True,
                "valid": True
            }
        )
    return header_list

def get_query_list(query):
    query_list = []
    for query_item in query:
        query_list.append(
            {
                "name": str(query_item[0]),
                "value": str(query_item[1]),
                "type": "text",
                "enable": True,
                "contentType": "text/plain",
                "encode": True,
                "file": False,
                "required": True,
                "valid": True
            }
        )
    return query_list


def get_assert_list(the_assert):
    assert_list = []
    for assert_item in the_assert:
        assert_list.append(
            {
                "type": "JSON",
                "expression": str(assert_item[0]),
                "expect": str(assert_item[1]),
                "description": str(assert_item[2])
            }
        )
    return assert_list

def get_extract_list(extract):
    extract_list = []
    for extract_item in extract:
        extract_list.append(
            {
                "type": "JSONPath",
                "variable": str(extract_item[0]),
                "value": "${"+str(extract_item[0])+"}",
                "expression": str(extract_item[1]),
                "multipleMatching": False,
                "valid": True
            }
        )
    return extract_list

def generate_request_item(name, url, method, query, header, body, the_assert, extract):
    query_list = get_query_list(query)

    raw = dict()
    for row_num in range(len(body)):
        body_row = body[row_num]
        raw[body_row[0]] = body_row[1]

    header_list = get_header_list(header)
    assert_list = get_assert_list(the_assert)
    extract_list = get_extract_list(extract)


    # itest平台原因，需要保持request顺序才能正常解析
    item = OrderedDict([
        ("type", "HTTP"),
        ("name", name),
        ("enable", True),
        ("assertions", {
            "jsonPath": assert_list
        }),
        ("extract", {
            "json": extract_list
        }),
        ("url", url),
        ("method", method),
        ("parameters", query_list),
        ("headers", header_list),
        ("body", {
            "type": "Raw",
            "raw": json.dumps(raw, indent=4, ensure_ascii=False),
            "kvs": [{
                "type": "text",
                "enable": True,
                "contentType": "text/plain",
                "encode": True,
                "file": False,
                "required": True,
                "valid": False
            }],
            "format": "json",
            "json": False,
            "kV": False,
            "oldKV": True,
            "valid": False,
            "xml": False
        })
    ])
    return item

def generate_require_items(requires_dir):
    require_items =[]
    result_json_array = read_require_config.read_excel(requires_dir)
    for row_num in range(0, len(result_json_array)):
        result_json = result_json_array[row_num]
        name = result_json['name']
        url = result_json['url']
        method = result_json['method']
        query = result_json['query']
        header = result_json['header']
        body = result_json['body']
        the_assert = result_json['normal_assert']
        extract = result_json['extract']

        require_item = generate_request_item(name, url, method, query, header, body, the_assert, extract)
        require_items.append(require_item)

    return require_items